"""
Script que genera una capa de puntos a partir de un fichero csv 
(municipiosAdemuz.csv) que se utiliza para trabajar con la herramienta 
miHerramientaDeInformacion.
"""

import os.path

from gvsiglib import *

def getMapContext():
  """
  Comprueba que el documanto activo es una vista y devuelve 
  el mapContext asociado a ella
  """
  view = gvSIG.getActiveDocument()
  if view == None:
    print "No se puede acceder al documento activo."
    return None
  try:
    mapContext = view.getModel().getMapContext()
  except Exception, e:
    print "El documento activo no parece ser una vista."
    print "Error %s %s" % (str(e.__class__),str(e))
    return None

  return mapContext

def isThereMyLayer(layers):
  """
  Funcion encargada de comprobar si la capa de trabajo 
  se encuentra en la coloeccion de capas indicada.
  """
  for n in range(layers.getLayersCount()):
    layer = layers.getLayer(n)
    if isinstance(layer,LayerCollection):
      if isThereMyLayer(layer):
        return True
    if layer.getProperty("capaConMiInformacionEspecialDeAdemuz")==1:
      return True
  return False

def createMyWorkLayer():
  """
  Funcion encargada de crear y cargar en la vista la capa 
  de trabajo a partir del fichero csv
  """
  mapContext = getMapContext()
  if mapContext==None:
    return
  
  #comprobamos si ya esta cargada la capa de trabajo en la vista
  layers=mapContext.getLayers()
  if isThereMyLayer(layers):
    return
  
  # Lo primero a hacer sera crear un dataSource basado en el fichero 
  # csv
  dataSourceFactory=LayerFactory.getDataSourceFactory()

  fileName = os.path.join(
      gvSIG.getScriptsDirectory(),
      "..",
      "..",
      "miHerramientaDeInformacion",
      "municipiosAdemuz.csv"
      )
  dataSourceFactory.addFileDataSource("csv string", "ademuz",fileName)
  ds = dataSourceFactory.createRandomDataSource("ademuz")
  ds.start()
  
  # Crearemos el driver que gestiona la capa de eventos y lo enlazaremos con 
  # la fuente de datos que acabamos de crear indicandole que columnas de esta 
  # representan los puntos de la geomretria
  xFieldIndex = ds.getFieldIndexByName("X")
  yFieldIndex = ds.getFieldIndexByName("Y")
  AddEventThemeDriver=gvSIG.classForName("com.iver.gvsig.addeventtheme.AddEventThemeDriver")
  addEventThemeDriver = AddEventThemeDriver()
  addEventThemeDriver.setData(ds, xFieldIndex, yFieldIndex)
  
  # Crearemos ahora la nueva capa basada en este driver
  capa = None
  try:
    myLayer=gvSIG.getExtensionPoints().get("Layers").create("com.iver.cit.gvsig.fmap.layers.FLayerGenericVectorial")
    myLayer.setName("ademuz")
    myLayer.setDriver(addEventThemeDriver)
    myLayer.setProjection(mapContext.getProjection())
  except Exception, e:
    print "Se ha producido un error creando la capa. Error %s %s" % (str(e.__class__),str(e))
    return
  
  # Una vez creada la capa se le añade una propiedad para reconocerla
  # como nuestra capa de trabajo
  myLayer.setProperty("capaConMiInformacionEspecialDeAdemuz",1)
  
  # La añadiremos a la lista de capas del mapContext de la vista
  mapContext.getLayers().addLayer(myLayer)
  
  # Indicamos al mapContext que se debe repintar
  mapContext.invalidate()

createMyWorkLayer() 
