from java.util import HashMap
from java.awt import Cursor, Point
from java.awt.event import MouseEvent

from gvsiglib import *

panel = None # Almacena una instancia de la ventana de informacion mostrada

class MyContentHandler(SaxContentHandler):
  """
  Parsea el xml asociado a la informacion de un punto
  transformandolo en un diccionario clave-valor

  FIXME: Solo funciona con capas vectoriales. Habria que arreglarlo
  para otro tipo de capas
  """
  def __init__(self, values):
    self.values = values

  def startElement(self, nameSpace, localName, qName, attrs):
    value = {} # Crea un diccionario vacio
    for i in range(attrs.getLength()):
      name=attrs.getQName(i)
      if name in ("",None):
        name=attrs.getLocalName(i)
      value[name] = attrs.getValue(i)
    if len(value) >0:
      self.values.append(value)

  def endElement(self, nameSpace, localName, qName):
    pass
  
  def characters(self, value, start, length):
    pass

class MyInformationToolListener(PointListener):
    """
    Esta clase recibe los eventos de click sobre el mapControl
    """
    def __init__(self,view,mapControl):
        self._cursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)
        self._mapControl=mapControl
        self._view=view
    
    def getCursor(self):
      "@sig public java.awt.Cursor getCursor()" 
      return self._cursor
    
    def cancelDrawing(self):
      "@sig public boolean cancelDrawing()"
      return False;
    
    def pointDoubleClick(self, event):
      "@sig public void pointDoubleClick(PointEvent event) throws BehaviorException"
      pass
    
    def point(self,event):
      "@sig public void point(PointEvent event) throws BehaviorException"
      global panel
      # Este evento es invocado cada vez que se produce un clic sobre el mapControl 
      # estando nuestra herramienta activa
      
      # Lo primero que haremos sera obtener la lista de capas activas en el TOC
      selectedLayers = self._mapControl.getMapContext().getLayers().getActives()
      
      # Si no hay ninguna capa seleccionada en el TOC no hacemos nada
      if len (selectedLayers)<1:
        return
      # Si hay mas de una capa activa presentaremos la ventana de informacion predeterminada
      if len(selectedLayers) >1:
        showInfo(self._view, event.getPoint())
        return
      # Si no esta activa la capa de trabajo presentaremos la ventana de informacion predeterminada
      if selectedLayers[0].getProperty("capaConMiInformacionEspecialDeAdemuz")!=1:
        showInfo(self._view, event.getPoint())
        return

      # Si hemos llegado hasta aqui es que solo estaba activa en el TOC nuestra capa de trabajo
      # procederemos a recuperar la informacion asociada al pùnto que se ha clicado
      tolerance = self._mapControl.getViewPort().toMapDistance(10) # Transforma pixels a unidades de mapa
      thePoint = Point(int(event.getPoint().getX()),int(event.getPoint().getY()))
      values = [] # Crea una lista vacia

      layer = selectedLayers[0]
      info = layer.getInfo(thePoint, tolerance, None)
      for atribute in info:
        atribute.parse(MyContentHandler(values))

      if len(values)<1:
        showMessageDialog("No hay informacion sobre el punto seleccionado")
        return
      
      # Una vez hemos recogido los atributos los presentamos usando el panel de informacion definido para ello
      params= HashMap()
      params.put("values", values)

      if panel != None:
        panel.close()
 
      panel=gvSIG.show("gvSIG/extensiones/miHerramientaDeInformacion/miPanelDeInformacion.xml","jython",325,150,params)
      
def showInfo(view,point):
  """
  Muestra la ventana de informacion predeterminada para el punto indicado de la vista
  """
  mapControl = view.getMapControl()
  infoListener = InfoListener(mapControl)
  event = MouseEvent(view,MouseEvent.BUTTON1,MouseEvent.ACTION_EVENT_MASK,MouseEvent.MOUSE_CLICKED,500,400,1,True)
  pointEvent = PointEvent(point,event)
  infoListener.point(pointEvent)


def main():
    view = gvSIG.getActiveDocument()
    if view == None:
      print "No se puede acceder al documento activo."
      return None
    try:
      mapContext = view.getModel().getMapContext()
      mapControl = view.getMapControl()

    except Exception, e:
      print "El documento activo no parece ser una vista."
      print "Error %s %s" % (str(e.__class__),str(e))
      return None
    if mapContext == None:
      return
    
    # Si no hemos registrado en el mapControl nuestra herramienta de informacion
    # Creamos nuestro Listener y lo registramos en el mapControl
    if not mapControl.hasTool("MyInformationToolListener"):
      il=MyInformationToolListener(view,mapControl)
      mapControl.addMapTool("MyInformationToolListener", PointBehavior(il))

    # Indicamos al mapControl que esta activa nuestra herramienta de informacion
    mapControl.setTool("MyInformationToolListener")
    
main()
 
